@@ -45,7 +45,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
       }
 
       if (!is.null(value.class) && !inherits(obj, value.class)) {
-        msg <- paste0("A query must result in a vector of class \"",
+        msg <- paste0("Filter must result in a vector of class \"",
                       value.class,
                       "\". The evaluated function is a vector of class \"",
                       class(obj), "\", please revise.")
@@ -54,7 +54,10 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
         return()
       }
 
-      rtn <<- list(fun=txt, class=class(obj)[1], summary=SummarizeVariable(obj),
+      summary.str <- paste(c("", capture.output(summary(obj)),
+                             "", capture.output(str(obj)),
+                             ""), collapse="\n")
+      rtn <<- list(fun=txt, class=class(obj)[1], summary=summary.str,
                    sample=na.omit(obj)[1])
     }
     tclvalue(tt.done.var) <- 1
@@ -63,9 +66,10 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   # Rebuild list box based on selected class type to show
   RebuildList <- function() {
     idx <- as.integer(tcl(frame1.box.3.1, "current"))
-    show.ids <- ids
     if (idx > 0)
-      show.ids <- ids[cls %in% classes[idx]]
+      show.ids <- ids[vapply(cols, function(i) classes[idx] %in% i$class, TRUE)]
+    else
+      show.ids <- ids
 
     tclvalue(variable.var) <- ""
     for (i in seq(along=show.ids))
@@ -124,11 +128,11 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   }
 
   # Call date and time format editor
-  CallFormatDateTime <- function() {
-    spec <- FormatDateTime(parent=tt)
+  CallFormatDateTime <- function(sample) {
+    fmt <- FormatDateTime(sample=sample, parent=tt)
     tkfocus(frame2.txt.2.1)
-    if(!is.null(spec))
-      InsertString(spec)
+    if(!is.null(fmt))
+      InsertString(gsub("%OS[[:digit:]]+", "%OS", fmt))
   }
 
   # Text edit functions
@@ -165,6 +169,8 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
 
   # Show unique values
   ShowUniqueValues <- function() {
+    tkconfigure(tt, cursor="watch")
+    on.exit(tkconfigure(tt, cursor="arrow"))
     idx <- as.integer(tkcurselection(frame1.lst.2.1))
     if (length(idx) == 0)
       return()
@@ -173,7 +179,6 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
 
     var.fmt <- cols[[idx]]$format
 
-    tkconfigure(tt, cursor="watch")
     var.vals <- unique(EvalFunction(cols[[idx]]$fun, cols))
     var.class <- cols[[idx]]$class
 
@@ -182,18 +187,16 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
       msg <- paste("There are", n, "unique values; this operation can be",
                    "computationally expensive. Would you like to continue?")
       ans <- as.character(tkmessageBox(icon="question", message=msg,
-                                       title="Warning", type="okcancel",
+                                       title="Warning", type="yesno",
                                        parent=tt))
-      if (ans == "cancel") {
-        tkconfigure(tt, cursor="arrow")
+      if (ans == "no")
         return()
-      }
     }
 
     var.vals <- sort(var.vals, na.last=TRUE)
     if (var.fmt == "") {
       var.vals.txt <- format(var.vals)
-    } else if (var.class == "POSIXct") {
+    } else if ("POSIXt" %in% var.class) {
       var.vals.txt <- format(var.vals, format=var.fmt)
     } else {
       var.vals.txt <- try(sprintf(var.fmt, var.vals), silent=TRUE)
@@ -208,7 +211,6 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
     tkselection.clear(frame1.lst.4.1, 0, "end")
     tkconfigure(frame1.but.5.1, state="disabled")
     tkfocus(frame2.txt.2.1)
-    tkconfigure(tt, cursor="arrow")
   }
 
   # Change variable selection
@@ -233,14 +235,14 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
     if (length(idx) == 0)
       return()
     val <- as.character(tkget(frame1.lst.4.1, idx, idx))
-    if (var.class == "factor" && is.na(suppressWarnings(as.numeric(val))))
+    if ("factor" %in% var.class && is.na(suppressWarnings(as.numeric(val))))
       var.class <- "character"
-    if (var.class == "POSIXct") {
+    if ("POSIXt" %in% var.class) {
       txt <- paste0("as.POSIXct(\"", val, "\", format = \"", var.fmt, "\")")
-    } else if (var.class == "integer" &&
+    } else if ("integer" %in% var.class &&
                !val %in% c("NA", "NaN", "Inf", "-Inf")) {
       txt <- paste0(val, "L")
-    } else if (var.class == "character" &&
+    } else if ("character" %in% var.class &&
                !val %in% c("NA", "NaN", "Inf", "-Inf")) {
       txt <- paste0("\"", val, "\"")
     } else {
@@ -272,23 +274,26 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
     old.fun <- cols[[as.integer(index)]]$fun
   }
   rtn <- NULL
-
   ids <- vapply(cols, function(i) i$id, "")
-  cls <- vapply(cols, function(i) i$class, "")
+
+  # Remove variable being defined
   if (!is.null(index)) {
     edit.fun.id <- ids[index]
     ids <- ids[-index]
-    cls <- cls[-index]
   }
 
+  # Class types
+  classes <- NULL
+  for (i in seq(along=cols)) {
+    if (!i %in% index)
+      classes <- c(classes, cols[[i]]$class)
+  }
+  classes <- suppressWarnings(sort(unique(classes)))
+
+  # Required vector length
   if (!is.null(value.length))
     value.length <- as.integer(value.length)
 
-  # Class types
-  classes <- c("numeric", "integer", "POSIXct", "logical",
-               "character", "factor")
-  classes <- classes[classes %in% cls]
-
   # Assign variables linked to Tk widgets
   variable.var <- tclVar()
   for (i in seq(along=ids))
@@ -303,7 +308,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
 
   tclServiceMode(FALSE)
 
-  tt <- tktoplevel(padx=0, pady=0)
+  tt <- tktoplevel()
   if (!is.null(parent)) {
     tkwm.transient(tt, parent)
     geo <- unlist(strsplit(as.character(tkwm.geometry(parent)), "\\+"))
@@ -336,23 +341,59 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   tkadd(menu.edit, "command", label="Clear console", accelerator="Ctrl+l",
         command=ClearConsole)
 
-  menu.class <- tkmenu(tt, tearoff=0)
-  tkadd(top.menu, "cascade", label="Class", menu=menu.class, underline=0)
+  menu.convert <- tkmenu(tt, tearoff=0)
+  tkadd(top.menu, "cascade", label="Convert", menu=menu.convert, underline=0)
 
-  tkadd(menu.class, "command", label="As character",
-        command=function() InsertString("as.character(<variable>)"))
-  tkadd(menu.class, "command", label="As factor",
+  menu.convert.char <- tkmenu(tt, tearoff=0)
+  tkadd(menu.convert.char, "command", label="Factor",
         command=function() InsertString("as.factor(<variable>)"))
-  tkadd(menu.class, "command", label="As integer",
-        command=function() InsertString("as.integer(<variable>)"))
-  tkadd(menu.class, "command", label="As logical",
-        command=function()  InsertString("as.logical(<variable>)"))
-  tkadd(menu.class, "command", label="As numeric",
+  tkadd(menu.convert.char, "command", label="Numeric",
         command=function() InsertString("as.numeric(<variable>)"))
-  tkadd(menu.class, "command", label="As POSIXct",
-        command=function() {
-          InsertString("as.POSIXct(<variable>, format = \"<format>\")")
-        })
+  tkadd(menu.convert.char, "command", label="Integer",
+        command=function() InsertString("as.integer(<variable>)"))
+  tkadd(menu.convert.char, "command", label="Logical",
+        command=function() InsertString("as.logical(<variable>)"))
+  tkadd(menu.convert.char, "command", label="POSIXct",
+        command=function() InsertString("as.POSIXct(strptime(<variable>, format = \"<format>\"))"))
+  tkadd(menu.convert.char, "command", label="Date",
+        command=function() InsertString("as.Date(<variable>, format = \"<format>\")"))
+  tkadd(menu.convert, "cascade", label="Character to", menu=menu.convert.char)
+
+  menu.convert.factor <- tkmenu(tt, tearoff=0)
+  tkadd(menu.convert.factor, "command", label="Character",
+        command=function() InsertString("as.character(<variable>)"))
+  tkadd(menu.convert.factor, "command", label="Integer",
+        command=function() InsertString("as.integer(<variable>)"))
+  tkadd(menu.convert, "cascade", label="Factor to", menu=menu.convert.factor)
+
+  menu.convert.num <- tkmenu(tt, tearoff=0)
+  tkadd(menu.convert.num, "command", label="POSIXct",
+        command=function() InsertString("as.POSIXct(<variable>, origin = \"1970-01-01 00:00:00.00\")"))
+  tkadd(menu.convert, "cascade", label="Numeric to", menu=menu.convert.num)
+
+  menu.convert.int <- tkmenu(tt, tearoff=0)
+  tkadd(menu.convert.int, "command", label="POSIXct",
+        command=function() InsertString("as.POSIXct(<variable>, origin = \"1970-01-01 00:00:00\")"))
+  tkadd(menu.convert.int, "command", label="Date",
+        command=function() InsertString("as.Date(<variable>, origin = \"1899-12-30\")"))
+  tkadd(menu.convert, "cascade", label="Integer to", menu=menu.convert.int)
+
+  menu.convert.log <- tkmenu(tt, tearoff=0)
+  tkadd(menu.convert.log, "command", label="Integer",
+        command=function() InsertString("as.integer(<variable>)"))
+  tkadd(menu.convert, "cascade", label="Logical to", menu=menu.convert.log)
+
+  menu.convert.posix <- tkmenu(tt, tearoff=0)
+  tkadd(menu.convert.posix, "command", label="Numeric",
+        command=function() InsertString("as.numeric(<variable>)"))
+  tkadd(menu.convert.posix, "command", label="Date",
+        command=function() InsertString("as.Date(<variable>, tz = \"UTC\")"))
+  tkadd(menu.convert, "cascade", label="POSIXct to", menu=menu.convert.posix)
+
+  menu.convert.date <- tkmenu(tt, tearoff=0)
+  tkadd(menu.convert.date, "command", label="Integer",
+        command=function() InsertString("as.integer(<variable>)"))
+  tkadd(menu.convert, "cascade", label="Date to", menu=menu.convert.date)
 
   menu.math <- tkmenu(tt, tearoff=0)
   tkadd(top.menu, "cascade", label="Math", menu=menu.math, underline=0)
@@ -490,17 +531,21 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
 
   menu.tools <- tkmenu(tt, tearoff=0)
   tkadd(top.menu, "cascade", label="Tools", menu=menu.tools, underline=0)
-  tkadd(menu.tools, "command", label="Build format for date-time variable\u2026",
-        command=CallFormatDateTime)
+  menu.tools.time <- tkmenu(tt, tearoff=0)
+  tkadd(menu.tools.time, "command", label="POSIXct\u2026",
+        command=function() CallFormatDateTime(Sys.time()))
+  tkadd(menu.tools.time, "command", label="Date\u2026",
+        command=function() CallFormatDateTime(Sys.Date()))
+  tkadd(menu.tools, "cascade", label="Build format for", menu=menu.tools.time)
 
   # Finalize top menu
   tkconfigure(tt, menu=top.menu)
 
   # Frame 0, ok and cancel buttons, and size grip
 
-  frame0 <- tkframe(tt, relief="flat", padx=0, pady=0)
+  frame0 <- tkframe(tt, relief="flat")
 
-  frame0a <- tkframe(frame0, relief="flat", padx=0, pady=0)
+  frame0a <- tkframe(frame0, relief="flat")
   frame0a.chk.1 <- ttkcheckbutton(frame0a, text="Inverse", variable=inverse.var)
   frame0a.chk.2 <- ttkcheckbutton(frame0a, text="Hyperbolic",
                                   variable=hyperbolic.var)
@@ -542,7 +587,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
 
   # Frame 1
 
-  frame1 <- tkframe(pw, relief="flat", padx=0, pady=0)
+  frame1 <- tkframe(pw, relief="flat")
 
   txt <- "Double click to insert variable"
   frame1.lab.1.1 <- ttklabel(frame1, text=txt, foreground="#141414")
@@ -592,7 +637,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
 
   # Frame 2
 
-  frame2 <- tkframe(pw, relief="flat", padx=0, pady=0)
+  frame2 <- tkframe(pw, relief="flat")
 
   txt <- "Define function"
   if (!is.null(index) && edit.fun.id != "")
@@ -611,7 +656,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   frame2.lab.1.2 <- ttklabel(frame2, text=txt, foreground="#A40802")
 
   frame2.txt.2.1 <- tktext(frame2, bg="white", font="TkFixedFont",
-                           padx=2, pady=2, width=50, height=12, undo=1,
+                           padx=2, pady=2, width=80, height=12, undo=1,
                            autoseparators=1, wrap="none", foreground="black",
                            relief="flat",
                            yscrollcommand=function(...)
@@ -620,7 +665,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
   frame2.ysc.2.2 <- ttkscrollbar(frame2, orient="vertical")
   tkconfigure(frame2.ysc.2.2, command=paste(.Tk.ID(frame2.txt.2.1), "yview"))
 
-  frame2a <- tkframe(frame2, relief="flat", padx=0, pady=0)
+  frame2a <- tkframe(frame2, relief="flat")
   frame2a.but.01 <- ttkbutton(frame2a, width=3, text="\u002b",
                               command=function() InsertString(" + "))
   frame2a.but.02 <- ttkbutton(frame2a, width=3, text="\u2212",
@@ -674,7 +719,7 @@ EditFunction <- function(cols, index=NULL, fun=NULL, value.length=NULL,
                    sticky="nsew")
   tkgrid.configure(frame2.ysc.2.2, padx=c(0, 10), pady=c(2, 0), columnspan=2,
                    sticky="ns")
-  tkgrid.configure(frame2a, pady=0, columnspan=2, sticky="we")
+  tkgrid.configure(frame2a, columnspan=2, sticky="we")
 
   tkgrid.rowconfigure(frame2, 1, weight=1)
   tkgrid.columnconfigure(frame2, 1, weight=1, minsize=20)
